#define F_CPU 4000000UL

#include<avr/io.h>
#include<util/delay.h>
#include<avr/interrupt.h>


#define F_OSC 4000000UL		           /* oscillator-frequency in Hz */
#define UART_BAUD_RATE 9600
#define UART_BAUD_CALC(UART_BAUD_RATE,F_OSC) ((F_OSC)/((UART_BAUD_RATE)*16l)-1)

void writeReg(unsigned char regName, unsigned char regValue);

/******************************************************************************************************/
void uart_init()
{	
	// set baud rate
	UBRRH = (uint8_t)(UART_BAUD_CALC(UART_BAUD_RATE,F_OSC) >> 8);
	UBRRL = (uint8_t) UART_BAUD_CALC(UART_BAUD_RATE,F_OSC);

	// Enable receiver and transmitter;
	UCSRB = (1 << RXEN) | (1 << TXEN);//  | (1 << RXCIE);

	//asynchronous 8N1
	UCSRC = (1 << URSEL) | (3 << UCSZ0);
}

/*******************************************************************************************************************/
void uart_putc(uint8_t c)
{
	// wait until UDR ready
	while (!(UCSRA & (1 << UDRE)));
	UDR = c;    // send character
}

/*******************************************************************************************************************/
void uart_puts (uint8_t * ch)
{
	while (*ch)
		uart_putc (*ch++);
}

/******************************************************************************************************/
void spi_init(void)
{
	DDRB |= (1<<PB7); //sck
	DDRB |= (1<<PB5); //mosi
	DDRB |= (1<<PB4); //cs
	
	PORTB |= (1<<PB6);

	//SPCR = (1<<SPE) | (1<<MSTR) | (1<<SPR0) | (1 << CPHA) | (1 << CPOL);//|(1<<DORD);

	// enable SPI, SPI Mastermode, SPI Signalform an LISLV02DQ angleichen, SPI Takt durch 128 teilen
	SPCR |= (1<<SPE) | (1<<MSTR) | (1<<CPOL) | (1<<CPHA) | (1<<SPR1) | (1<<SPR0);
	SPCR &= ~(1<<DORD);// MSB zuerst bertragen
}

/******************************************************************************************************/
void spi_trans(unsigned char cData)
{
	SPDR = cData; 									// Write Data to register

	while(!(SPSR & (1<<SPIF))); 					// Wait for transmission to complete

	SPSR &= ~(1 << 7);
}

/******************************************************************************************************/
unsigned char spi_rece(void)
{
	 //Wait for reception complete 
	while(!(SPSR & (1<<SPIF)));
	//Return data register 
	return SPDR;
}

/******************************************************************************************************/
void accInit(void)
{
	unsigned char address = 0;
	unsigned char data	 = 0;

	address = 0x20;					// LIS3L_CTRL_REG1
	data    = 0xC7;					// enable device including all axes

	spi_trans (address);
	spi_trans (data);
}

/******************************************************************************************************/
unsigned char accReadWhoAmI ()
{
	spi_trans (0x0F);		// Transmit WhoAmI 

	return spi_rece();
}

/******************************************************************************************************/
unsigned char USART_Receive(void)
{
	while(!(UCSRA & (1<<RXC))); 		//wait for data to be received
	
	return UDR;
}

/******************************************************************************************************/
void wait (unsigned long ws)
{
	while (ws--)
		asm volatile ("nop");
}

/******************************************************************************************************/
int main(void)
{
	unsigned char readData = 0x00;

	uart_init();

	uart_puts ((unsigned char *) "LIS3LV02DQ Testing\n");

	spi_init();

	accInit();

	while(1)
	{
		readData = accReadWhoAmI ();
		uart_putc (readData);
		uart_putc ('\n');
		wait (100000);
	}
	
}
